<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://http://designingmedia.com/
 * @since      1.0.0
 *
 * @package    Akd_Framework
 * @subpackage Akd_Framework/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Akd_Framework
 * @subpackage Akd_Framework/admin
 * @author     Ammar Nasir <info@domain.com>
 */
class Akd_Framework_Admin {

	use Akd_Theme_Activation;
	use Akd_Required_plugin_Activation;

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	private $activated_theme;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->activated_theme = wp_get_theme();

	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Akd_Framework_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Akd_Framework_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/akd-framework-admin.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Akd_Framework_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Akd_Framework_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/akd-framework-admin.js', array( 'jquery' ), $this->version, false );
		wp_enqueue_script( 'merlin', plugins_url() .'/akd-demo-importer/includes/merlin/assets/js/merlin.js', array( 'jquery' ), $this->version );

		// Localize the script with URLs
		$urls = [
			'siteUrl' => site_url(),
			'ajaxUrl' => admin_url('admin-ajax.php'),
		];

		$elementor = false;
		// Check if Elementor plugin is active
		if (in_array('elementor/elementor.php', apply_filters('active_plugins', get_option('active_plugins')))) {
			$elementor = true;
		}

		$additional_data = [];
		if(get_option('_pruchase_code') && get_option('_encryption_key')){
			$additional_data = [
				'purchaseCode' => $this->akd_decrypt_data( get_option('_pruchase_code'), get_option('_encryption_key') ),
			];
		}

		// Check if TGMPA is loaded (optional)
		if (class_exists('TGM_Plugin_Activation')) {
			// Check if TGMPA is initialized
			if (isset($GLOBALS['tgmpa'])) {
				// Get TGMPA instance
				$tgmpa_instance = $GLOBALS['tgmpa'];
				$required_plugins = '';
				if(TGM_Plugin_Activation::get_instance()->is_tgmpa_complete() == true){
					$required_plugins = 'installed';
				}
			}
		}

		$child_theme_directory = get_theme_root() . '/'.lcfirst($this->activated_theme->get('Name')).'-child'; // Path to the child theme director
		$theme_info = [
			'themeName' => $this->activated_theme->get('Name'),
			'childTheme' => $this->activated_theme->get('Template'),
			'elementor' => $elementor,
			'requiredPlugins' => $required_plugins,
			'childThemeFound' => is_dir($child_theme_directory)
		];

		global $wpdb;

		// Create an array to store all the information
		$server_info = [
			'wp_version' 			 => get_bloginfo('version'),
			'wp_dir_writeable' 		 => is_writable(wp_upload_dir()['basedir']),
			'php_v' 				 => phpversion(),
			'php_memory_limit' 		 => ini_get('memory_limit'),
			'php_post_max_size' 	 => ini_get('post_max_size'),
			'php_max_upload_size' 	 => ini_get('upload_max_filesize'),
			'php_max_input_vars' 	 => ini_get('max_input_vars'),
			'php_max_execution_time' => ini_get('max_execution_time'),
			'wp_memory_limit' 		 => WP_MEMORY_LIMIT,
			'mysql_v' 				 => $wpdb->db_version()
		];


		// Combine localization data into a single array
		$localization_data = [
			'urls_to_frontend'    => $urls,
			'additional_data'	  => $additional_data,
			'theme_info'		  => $theme_info,
			'server_info'		  => $server_info,
			'theme_demo' 		  => get_option('_akd_demo_import') ? get_option('_akd_demo_import') : '',
			'valid_purchase_code' => get_option('_purchase_code_validate') ? get_option('_purchase_code_validate') : ''
		];

		// Localize the script with combined data
		wp_localize_script($this->plugin_name, 'localization_data', $localization_data);

		wp_localize_script(
			'merlin', 'merlin_params', array(
				'tgm_plugin_nonce' => array(
					'update'  => wp_create_nonce( 'tgmpa-update' ),
					'install' => wp_create_nonce( 'tgmpa-install' ),
				),
				'ajaxurl'          => admin_url( 'admin-ajax.php' ),
				'wpnonce'          => wp_create_nonce( 'merlin_nonce' ),
			)
		);

	}

	public function pre($data){
		echo '<pre>'; print_r($data);
	}

	/**
	 * Method to trigger admin_init hook methods
	 */
	public function akd_admin_init_callback(){
		//method to remove GDPR cookie compliance warnings from the merlin page
		$this->akd_remove_cookie_compliance_warnings();

		//method to get and save elementor global settings within wp_options
		$this->akd_get_elementor_global_settings();

	}

	/**
	 * Method to remove GDPR cookie compliance warnings from the merlin page
	 */
	private function akd_remove_cookie_compliance_warnings(){
		//cookie complaince plugin warning on merlin page
		if ( is_plugin_active( 'gdpr-cookie-compliance/moove-gdpr.php' ) ) {
			if(isset($_GET['page']) && $_GET['page'] === 'merlin'){
				$this->remove_class_action('gdpr_check_review_banner_condition', 'Moove_GDPR_Review', 'gdpr_check_review_banner_condition_func');
			}
		}
	}

	/**
	 * Method to import demo content files
	 */
	public function akd_import_theme_demo_files() {

		return array(
             
			array(
				'import_file_name' => 'Energox',
				'categories' => array() ,
				'local_import_file'          => trailingslashit(plugin_dir_path( dirname( __FILE__ ) )).'admin/demo/energox-demo-content.xml',
				'local_import_widget_file'   => trailingslashit(plugin_dir_path( dirname( __FILE__ ) )).'admin/demo/site-settings.json',
				'local_import_customizer_file' => trailingslashit(plugin_dir_path( dirname( __FILE__ ) )).'admin/demo/energoxcustomizer.dat',        
				'import_preview_image_url'   => trailingslashit(plugin_dir_path( dirname( __FILE__ ) )).'admin/demo/screenshot.png',
			),

		);

	}

	/**
	 * Method to remove specific action or filter
	 */
	private function remove_class_action ($action,$class,$method) {
		global $wp_filter ;
		if (isset($wp_filter[$action])) {
			$len = strlen($method) ;
			foreach ($wp_filter[$action] as $pri => $actions) {
				foreach ($actions as $name => $def) {
					if (substr($name,-$len) == $method) {
						if (is_array($def['function'])) {
							if (get_class($def['function'][0]) == $class) {
								if (is_object($wp_filter[$action]) && isset($wp_filter[$action]->callbacks)) {
									unset($wp_filter[$action]->callbacks[$pri][$name]) ;
								} else {
									unset($wp_filter[$action][$pri][$name]) ;
								}
							}
						}
					}
				}
			}
		}
	}

	/**
	 * Method to get elementor global site settings
	 */
	private function akd_get_elementor_global_settings(){
		// Check if Elementor plugin is active
		if (in_array('elementor/elementor.php', apply_filters('active_plugins', get_option('active_plugins')))) {
			if(isset($_POST['akd-framework-admin-page'])){
				if(is_array($_POST['akd-framework-admin-page']['akd-theme-dashboard'])){

					global $wpdb;

					$active_kit_id = get_option('elementor_active_kit');
					if($active_kit_id){
						$post_meta = get_post_meta($active_kit_id, '_elementor_page_settings', true);

						if($post_meta){
							$akd_theme_system_colors = $_POST['akd-framework-admin-page']['akd-theme-dashboard']['akd-theme-colors'];
							$akd_theme_custom_colors = $_POST['akd-framework-admin-page']['akd-theme-dashboard']['akd-custom-colors'];
							$akd_theme_typography = $_POST['akd-framework-admin-page']['akd-theme-dashboard'];
							
							//typography update
							$post_meta['system_typography'][0]['typography_font_family'] = $akd_theme_typography['akd-primary-typography']; 
							$post_meta['system_typography'][0]['typography_font_weight'] = $akd_theme_typography['akd-primary-font-weight']; 
							$post_meta['system_typography'][1]['typography_font_family'] = $akd_theme_typography['akd-secondary-typography'];
							$post_meta['system_typography'][1]['typography_font_weight'] = $akd_theme_typography['akd-secondary-font-weight']; 
							$post_meta['system_typography'][2]['typography_font_family'] = $akd_theme_typography['akd-text-typography'];
							$post_meta['system_typography'][2]['typography_font_weight'] = $akd_theme_typography['akd-text-font-weight'];
							
							// Update colors based on second array
							foreach ($post_meta['custom_colors'] as &$color) {
								// Extract the ID from the _id field
								$id = $color['_id'];
								
								// Check if the corresponding custom color exists
								$custom_color_key = 'custom-color-' . $id;
								if (array_key_exists($custom_color_key, $akd_theme_custom_colors)) {
									// Update the color value
									$color['color'] = $akd_theme_custom_colors[$custom_color_key];
								}
							}

							$post_meta['system_colors'][0]['color'] = $akd_theme_system_colors['primary'] ? $akd_theme_system_colors['primary'] : '';
							$post_meta['system_colors'][1]['color'] = $akd_theme_system_colors['secondary'] ? $akd_theme_system_colors['secondary'] : '';
							$post_meta['system_colors'][2]['color'] = $akd_theme_system_colors['text'] ? $akd_theme_system_colors['text'] : '';
							$post_meta['system_colors'][3]['color'] = $akd_theme_system_colors['accent'] ? $akd_theme_system_colors['accent'] : '';

							update_post_meta($active_kit_id, '_elementor_page_settings', $post_meta);

							$active_kit = \Elementor\Plugin::$instance->kits_manager->get_active_kit_for_frontend();
							if($active_kit){

								$query = $wpdb->prepare("
									SELECT *
									FROM $wpdb->postmeta
									WHERE 
										CONVERT(meta_id USING utf8) LIKE %s OR
										CONVERT(post_id USING utf8) LIKE %s OR
										CONVERT(meta_key USING utf8) LIKE %s OR
										CONVERT(meta_value USING utf8) LIKE %s
								", '%_elementor_page_settings%', '%_elementor_page_settings%', '%_elementor_page_settings%', '%_elementor_page_settings%');

								$results = $wpdb->get_results($query);

								// Handle results
								if ($results) {
									foreach ($results as $key => $result) {
										if($key == 0){
											$elementor_post_data = get_post_meta( $result->post_id, '_elementor_page_settings', true );
											$elementor_post_data['system_colors'][0]['color'] = $akd_theme_system_colors['primary'] ? $akd_theme_system_colors['primary'] : '';
											$elementor_post_data['system_colors'][1]['color'] = $akd_theme_system_colors['secondary'] ? $akd_theme_system_colors['secondary'] : '';
											$elementor_post_data['system_colors'][2]['color'] = $akd_theme_system_colors['text'] ? $akd_theme_system_colors['text'] : '';
											$elementor_post_data['system_colors'][3]['color'] = $akd_theme_system_colors['accent'] ? $akd_theme_system_colors['accent'] : '';
											// $elementor_post_data['custom_colors'][5]['color'] = '#dd3333';

											$all_settings = $active_kit->get_settings();
											$all_settings['system_colors'][0]['color'] = $akd_theme_system_colors['primary'] ? $akd_theme_system_colors['primary'] : '';
											$all_settings['system_colors'][1]['color'] = $akd_theme_system_colors['secondary'] ? $akd_theme_system_colors['secondary'] : '';
											$all_settings['system_colors'][2]['color'] = $akd_theme_system_colors['text'] ? $akd_theme_system_colors['text'] : '';
											$all_settings['system_colors'][3]['color'] = $akd_theme_system_colors['accent'] ? $akd_theme_system_colors['accent'] : '';
											// $all_settings['custom_colors'][5]['color'] = '#dd3333';
											// $all_settings['site_name'] = 'New Site Title';
											$active_kit->set_settings($all_settings);
											$active_kit->save($all_settings);

											// update_post_meta( 9057, '_elementor_page_settings', $elementor_post_data );
											$elementor_global_settings = json_encode($active_kit->get_settings(), true);
											update_option( '_akd_elementor_global', $elementor_global_settings );
										}
									}
								}
							}
						}
					}
				}
			}

			// $postMeta['custom_colors'][5]['color'] = '#dd3333';
			// Elementor is active, proceed to retrieve global theme settings
			// $elementor_theme_settings = \Elementor\Core\Settings\Manager::get_settings_managers('general')->get_model()->get_settings();
			
		}
	}

	/**
	 * Method to update elementor global site settings
	 */
	public function akd_update_elementor_global_settings($options){

	}

}
